﻿using System;
using CocosSharp;
using CocosDenshion;

namespace KuruKuru
{
	public class GameLayer : BaseLayer
	{
		CCSprite _stump;
		CCSprite[] _ladybug = new CCSprite[9];
		CCPoint[] _point = { new CCPoint(-150,-150), new CCPoint(150,-150), new CCPoint(0,250),
			new CCPoint(150,150), new CCPoint(-150,150),new CCPoint(0,0),
			new CCPoint(-250,0), new CCPoint(250,0),  new CCPoint(0,-250)};
		float[] _rotation = new float[9] { 0, 40, 80, 120, 160, 200, 240, 280, 320 };
		float[] _delta = new float[9];
		bool _touch;
		static int _stage = 3;
		float _scale = 1;

		public GameLayer()
			: base()
		{
			Color = new CCColor3B(0, 160, 0);
			Opacity = 255;

			var listener = new CCEventListenerTouchOneByOne();
			listener.OnTouchBegan = CCEventListener_TouchBegan;
			AddEventListener(listener, this);
			listener.OnTouchEnded = CCEventListener_TouchEnded;
			AddEventListener(listener, this);

			_touch = false;
			initSounds();
		}

		private void initSounds()
		{
			CCSimpleAudioEngine.SharedEngine.PreloadEffect("Sounds/GameStart");
			CCSimpleAudioEngine.SharedEngine.PreloadEffect("Sounds/Collide");
			CCSimpleAudioEngine.SharedEngine.EffectsVolume = 1;
			CCSimpleAudioEngine.SharedEngine.PlayEffect("GameStart", false);
		}

		protected override void AddedToScene()
		{
			base.AddedToScene();
			_stump = new CCSprite("Images/Stump.png", null);
			_stump.Position = new CCPoint(ContentSize.Center.X, ContentSize.Center.Y);
			AddChild(_stump);
			_ladybug[0] = new CCSprite("Images/Ladybug.png", null);
			_ladybug[0].Tag = 0;
			for (var i = 1; i < _stage; i++)
			{
				_ladybug[i] = new CCSprite(_ladybug[0].Texture, null);
				_ladybug[i].Tag = i;
			}
			initStage();
			startScheduling();
		}

		private void initStage()
		{
			for (var i = 0; i < _stage; i++)
			{
				_ladybug[i].Visible = true;
				_ladybug[i].Position = new CCPoint(ContentSize.Center.X, ContentSize.Center.Y) + _point[i];
				_delta[i] = (float)(((CCRandom.Next(2) == 0) ? -1 : 1) * (2 + 5 * CCRandom.NextDouble()));
				AddChild(_ladybug[i]);
			}
			_scale = 1;
		}

		private void startScheduling()
		{
			Schedule(t => updateLadybug(), 0.03f);
		}

		private void updateLadybug()
		{
			int left = 0;
			_scale -= 0.0002f;
			_stump.Scale = _scale;
			for (int i = 0; i < _stage; i++)
			{
				var bug = _ladybug[i];
				if (!bug.Visible) continue;
				left++;
				bug.Rotation = _rotation[i];
				if (_touch)
				{
					var radian = CCMathHelper.ToRadians(_rotation[i]);
					bug.PositionX += CCMathHelper.Cos(radian) * 5;
					bug.PositionY -= CCMathHelper.Sin(radian) * 5;
					for (int j = 0; j < _stage; j++)
					{
						var bug2 = _ladybug[j];
						if (i == j || !bug2.Visible) continue;
						if (Math.Abs(bug.PositionX - bug2.PositionX) < 50
						    &&  Math.Abs(bug.PositionY - bug2.PositionY) < 50 )
						{
							RemoveChild(bug2);
							bug2.Visible = false;
							CCSimpleAudioEngine.SharedEngine.PlayEffect("Collide", false);
							break;
						}
					}
				}
				else
				{
					_rotation[i] += _delta[i];
					if (_rotation[i] < 0)
						_rotation[i] += 360;
					if (_rotation[i] >= 360)
						_rotation[i] -= 360;
				}
				float x = bug.PositionX - ContentSize.Center.X;
				float y = bug.PositionY - ContentSize.Center.Y;
				if (x * x + y * y > 320 * _scale * 320 * _scale)
				{
					if (_stage > 2) _stage--;
					ReplaceScene(new GameOverLayer());
					return;
				}
			}
			if (left <= 1)
			{
				if (_stage < 9)
				{
					_stage++;
					ReplaceScene(new StageClearLayer());
				}
				else
				{
					_stage = 3;
					ReplaceScene(new GameClearLayer());
				}
			}
		}

		protected bool CCEventListener_TouchBegan(CCTouch touch, CCEvent touchEvent)
		{
			_touch = true;
			return true;
		}

		protected void CCEventListener_TouchEnded(CCTouch touch, CCEvent touchEvent)
		{
			_touch = false;
		}

		private void ReplaceScene(BaseLayer layer)
		{
			var newScene = new CCScene(this.Window);
			newScene.AddChild(layer);
			CCTransitionScene cCTransitionScene = new CCTransitionFade(1.0f, newScene);
			Director.ReplaceScene(cCTransitionScene);
			base.OnExit();
		}
	}
}
